@@ -10,10 +10,12 @@
 # pylint: disable=import-error
 # pylint: disable=pointless-string-statement
 # pylint: disable=invalid-name
+# pylint: disable=unused-import
+# flake8: noqa: F401
 
 """This file includes the python code of BSTLD.rst and read_dataset_class.rst."""
 
-"""Authorize a Client Object"""
+"""Authorize a Client Instance"""
 from tensorbay import GAS
 
 ACCESS_KEY = "Accesskey-*****"
@@ -24,20 +26,21 @@ gas = GAS(ACCESS_KEY)
 gas.create_dataset("BSTLD")
 """"""
 
-"""List Dataset Names"""
-list(gas.list_dataset_names())
+"""Organize Dataset / regular import"""
+from tensorbay.dataset import Data, Dataset
+from tensorbay.label import LabeledBox2D
+
 """"""
 
-"""Read Dataset Class / organize dataset"""
+"""Organize dataset / import dataloader"""
 from tensorbay.opendataset import BSTLD
 
 dataset = BSTLD("path/to/dataset/directory")
 """"""
 
 """Upload Dataset"""
-# dataset is the one you initialized in "Organize Dataset" section
-dataset_client = gas.upload_dataset(dataset, jobs=8, skip_uploaded_files=False)
-dataset_client.commit("BSTLD")
+dataset_client = gas.upload_dataset(dataset)
+dataset_client.commit("initial commit")
 """"""
 
 """Read Dataset / get dataset"""
@@ -45,7 +48,7 @@ dataset_client = gas.get_dataset("BSTLD")
 """"""
 
 """Read Dataset / list segment names"""
-list(dataset_client.list_segment_names())
+dataset_client.list_segment_names()
 """"""
 
 """Read Dataset Class / get a segment"""
