@@ -23,14 +23,15 @@ Please refer to :class:`~tensorbay.dataset.dataset.FusionDataset` for more infor
 """
 
 import sys
-from typing import TYPE_CHECKING, Any, Dict, Iterator, Optional, Tuple
+from typing import TYPE_CHECKING, Any, Dict, Generator, Iterable, Iterator, Optional, Tuple, Union
 
 from ..dataset import Data, Frame, FusionSegment, Notes, Segment
+from ..exception import FrameError
 from ..label import Catalog
-from ..utility import Deprecated
+from ..utility import Deprecated, KwargsDeprecated
 from .commit_status import CommitStatus
 from .exceptions import GASSegmentError
-from .requests import Client, multithread_upload, paging_range
+from .requests import Client, PagingList, Tqdm, multithread_upload
 from .segment import FusionSegmentClient, SegmentClient
 from .struct import Branch, Commit, Draft, Tag
 
@@ -51,6 +52,11 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
         dataset_id: Dataset ID.
         gas_client: The initial client to interact between local and TensorBay.
 
+    Attributes:
+        name: Dataset name.
+        dataset_id: Dataset ID.
+        status: The status of the dataset client.
+
     """
 
     _client: Client
@@ -88,82 +94,58 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
         response = self._client.open_api_do("POST", "drafts", self.dataset_id, json=post_data)
         return response.json()["draftNumber"]  # type: ignore[no-any-return]
 
-    def _list_drafts(
-        self, *, start: int = 0, stop: int = sys.maxsize, page_size: int = 128
-    ) -> Iterator[Draft]:
-        params: Dict[str, Any] = {}
+    def _generate_drafts(self, offset: int = 0, limit: int = 128) -> Generator[Draft, None, int]:
+        params = {"offset": offset, "limit": limit}
+        response = self._client.open_api_do("GET", "drafts", self.dataset_id, params=params).json()
 
-        for params["offset"], params["limit"] in paging_range(start, stop, page_size):
-            response = self._client.open_api_do(
-                "GET", "drafts", self.dataset_id, params=params
-            ).json()
-            for draft_info in response["drafts"]:
-                yield Draft.loads(draft_info)
-            if response["recordSize"] + response["offset"] >= response["totalCount"]:
-                break
+        for item in response["drafts"]:
+            yield Draft.loads(item)
 
-    def _list_commits(
-        self,
-        revision: Optional[str] = None,
-        *,
-        start: int = 0,
-        stop: int = sys.maxsize,
-        page_size: int = 128,
-    ) -> Iterator[Commit]:
-        params: Dict[str, Any] = {}
+        return response["totalCount"]  # type: ignore[no-any-return]
+
+    def _generate_commits(
+        self, revision: Optional[str] = None, offset: int = 0, limit: int = 128
+    ) -> Generator[Commit, None, int]:
+        params: Dict[str, Any] = {"offset": offset, "limit": limit}
         if revision:
             params["commit"] = revision
 
-        for params["offset"], params["limit"] in paging_range(start, stop, page_size):
-            response = self._client.open_api_do(
-                "GET", "commits", self.dataset_id, params=params
-            ).json()
-            for commit_info in response["commits"]:
-                yield Commit.loads(commit_info)
-            if response["recordSize"] + response["offset"] >= response["totalCount"]:
-                break
+        response = self._client.open_api_do("GET", "commits", self.dataset_id, params=params).json()
 
-    def _list_tags(
-        self,
-        name: Optional[str] = None,
-        *,
-        start: int = 0,
-        stop: int = sys.maxsize,
-        page_size: int = 128,
-    ) -> Iterator[Tag]:
-        params: Dict[str, Any] = {}
+        for item in response["commits"]:
+            yield Commit.loads(item)
+
+        return response["totalCount"]  # type: ignore[no-any-return]
+
+    def _generate_tags(
+        self, name: Optional[str] = None, offset: int = 0, limit: int = 128
+    ) -> Generator[Tag, None, int]:
+        params: Dict[str, Any] = {"offset": offset, "limit": limit}
         if name:
             params["name"] = name
 
-        for params["offset"], params["limit"] in paging_range(start, stop, page_size):
-            response = self._client.open_api_do(
-                "GET", "tags", self.dataset_id, params=params
-            ).json()
-            for tag_info in response["tags"]:
-                yield Tag.loads(tag_info)
-            if response["recordSize"] + response["offset"] >= response["totalCount"]:
-                break
+        response = self._client.open_api_do("GET", "tags", self.dataset_id, params=params).json()
 
-    def _list_branches(
-        self,
-        name: Optional[str] = None,
-        *,
-        start: int = 0,
-        stop: int = sys.maxsize,
-        page_size: int = 128,
-    ) -> Iterator[Branch]:
-        params: Dict[str, Any] = {}
+        for item in response["tags"]:
+            yield Tag.loads(item)
+
+        return response["totalCount"]  # type: ignore[no-any-return]
+
+    def _generate_branches(
+        self, name: Optional[str] = None, offset: int = 0, limit: int = 128
+    ) -> Generator[Branch, None, int]:
+        params: Dict[str, Any] = {"offset": offset, "limit": limit}
         if name:
             params["name"] = name
 
-        for params["offset"], params["limit"] in paging_range(start, stop, page_size):
-            response = self._client.open_api_do(
-                "GET", "branches", self.dataset_id, params=params
-            ).json()
-            for branch_info in response["branches"]:
-                yield Branch.loads(branch_info)
-            if response["recordSize"] + response["offset"] >= response["totalCount"]:
-                break
+        response = self._client.open_api_do(
+            "GET", "branches", self.dataset_id, params=params
+        ).json()
+
+        for item in response["branches"]:
+            yield Branch.loads(item)
+
+        return response["totalCount"]  # type: ignore[no-any-return]
 
     def _create_segment(self, name: str) -> None:
         post_data: Dict[str, Any] = {"name": name}
@@ -171,18 +153,21 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
 
         self._client.open_api_do("POST", "segments", self.dataset_id, json=post_data)
 
-    def _list_segments(
-        self, *, start: int = 0, stop: int = sys.maxsize, page_size: int = 128
-    ) -> Iterator[Dict[str, str]]:
+    def _generate_segment_names(
+        self, offset: int = 0, limit: int = 128
+    ) -> Generator[str, None, int]:
         params: Dict[str, Any] = self._status.get_status_info()
+        params["offset"] = offset
+        params["limit"] = limit
 
-        for params["offset"], params["limit"] in paging_range(start, stop, page_size):
-            response = self._client.open_api_do(
-                "GET", "segments", self.dataset_id, params=params
-            ).json()
-            yield from response["segments"]
-            if response["recordSize"] + response["offset"] >= response["totalCount"]:
-                break
+        response = self._client.open_api_do(
+            "GET", "segments", self.dataset_id, params=params
+        ).json()
+
+        for item in response["segments"]:
+            yield item["name"]
+
+        return response["totalCount"]  # type: ignore[no-any-return]
 
     @property
     def name(self) -> str:
@@ -250,13 +235,34 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
         if not draft_number:
             raise TypeError("The given draft number is illegal")
 
-        for draft in self._list_drafts():
+        for draft in self.list_drafts():
             if draft_number == draft.number:
                 return draft
 
         raise TypeError(f"The draft: {draft_number} does not exist.")
 
-    @Deprecated(since="1.2.0", removed_in="1.5.0", substitute="DatasetClientBase.list_draft")
+    @KwargsDeprecated(
+        ("start", "stop"),
+        since="1.3.0",
+        removed_in="1.5.0",
+        substitute="PagingList[start:stop]",
+    )
+    def list_drafts(self, **kwargs: int) -> PagingList[Draft]:
+        """List all the drafts.
+
+        Arguments:
+            kwargs: For deprecated keyword arguments: "start" and "stop".
+
+        Returns:
+            The PagingList of :class:`drafts<.Draft>`.
+
+        """
+        start = kwargs.get("start", 0)
+        stop = kwargs.get("stop", sys.maxsize)
+
+        return PagingList(self._generate_drafts, 128, slice(start, stop))
+
+    @Deprecated(since="1.2.0", removed_in="1.5.0", substitute=list_drafts)
     def list_draft_titles_and_numbers(
         self, *, start: int = 0, stop: int = sys.maxsize
     ) -> Iterator[Dict[str, Any]]:
@@ -270,24 +276,11 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
             The dict containing title and number of drafts.
 
         """
-        for draft in self._list_drafts(start=start, stop=stop):
+        for draft in self.list_drafts(start=start, stop=stop):
             yield draft.dumps()
 
-    def list_drafts(self, *, start: int = 0, stop: int = sys.maxsize) -> Iterator[Draft]:
-        """List all the drafts.
-
-        Arguments:
-            start: The index to start.
-            stop: The index to end.
-
-        Yields:
-            The :class:`drafts<.Draft>`.
-
-        """
-        yield from self._list_drafts(start=start, stop=stop)
-
     def get_commit(self, revision: Optional[str] = None) -> Commit:
-        """Get the certain commit with the given commit key.
+        """Get the certain commit with the given revision.
 
         Arguments:
             revision: The information to locate the specific commit, which can be the commit id,
@@ -309,15 +302,19 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
             raise TypeError("The given revision is illegal")
 
         try:
-            commit = next(self._list_commits(revision))
+            commit = next(self._generate_commits(revision))
         except StopIteration as error:
             raise TypeError(f"The commit: {revision} does not exist.") from error
 
         return commit
 
-    def list_commits(
-        self, revision: Optional[str] = None, *, start: int = 0, stop: int = sys.maxsize
-    ) -> Iterator[Commit]:
+    @KwargsDeprecated(
+        ("start", "stop"),
+        since="1.3.0",
+        removed_in="1.5.0",
+        substitute="PagingList[start:stop]",
+    )
+    def list_commits(self, revision: Optional[str] = None, **kwargs: int) -> PagingList[Commit]:
         """List the commits.
 
         Arguments:
@@ -325,16 +322,20 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
                 the branch name, or the tag name.
                 If is given, list the commits before the given commit.
                 If is not given, list the commits before the current commit.
-            start: The index to start.
-            stop: The index to end.
+            kwargs: For deprecated keyword arguments: "start" and "stop".
 
-        Yields:
-            The :class:`tags<.Commit>`.
+        Returns:
+            The PagingList of :class:`commits<.Commit>`.
 
         """
-        if not revision:
-            revision = self._status.commit_id
-        yield from self._list_commits(revision, start=start, stop=stop)
+        start = kwargs.get("start", 0)
+        stop = kwargs.get("stop", sys.maxsize)
+
+        return PagingList(
+            lambda offset, limit: self._generate_commits(revision, offset, limit),
+            128,
+            slice(start, stop),
+        )
 
     def create_tag(self, name: str, revision: Optional[str] = None) -> None:
         """Create the tag for a commit.
@@ -371,24 +372,36 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
             raise TypeError("The given tag name is illegal")
 
         try:
-            tag = next(self._list_tags(name))
+            tag = next(self._generate_tags(name))
         except StopIteration as error:
             raise TypeError(f"The tag: {name} does not exist.") from error
 
         return tag
 
-    def list_tags(self, *, start: int = 0, stop: int = sys.maxsize) -> Iterator[Tag]:
+    @KwargsDeprecated(
+        ("start", "stop"),
+        since="1.3.0",
+        removed_in="1.5.0",
+        substitute="PagingList[start:stop]",
+    )
+    def list_tags(self, **kwargs: int) -> PagingList[Tag]:
         """List the information of tags.
 
         Arguments:
-            start: The index to start.
-            stop: The index to end.
+            kwargs: For deprecated keyword arguments: "start" and "stop".
 
-        Yields:
-            The :class:`tags<.Tag>`.
+        Returns:
+            The PagingList of :class:`tags<.Tag>`.
 
         """
-        yield from self._list_tags(start=start, stop=stop)
+        start = kwargs.get("start", 0)
+        stop = kwargs.get("stop", sys.maxsize)
+
+        return PagingList(
+            lambda offset, limit: self._generate_tags(None, offset, limit),
+            128,
+            slice(start, stop),
+        )
 
     def get_branch(self, name: str) -> Branch:
         """Get the branch with the given name.
@@ -407,24 +420,36 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
             raise TypeError("The given branch name is illegal")
 
         try:
-            branch = next(self._list_branches(name))
+            branch = next(self._generate_branches(name))
         except StopIteration as error:
             raise TypeError(f"The branch: {name} does not exist.") from error
 
         return branch
 
-    def list_branches(self, *, start: int = 0, stop: int = sys.maxsize) -> Iterator[Branch]:
+    @KwargsDeprecated(
+        ("start", "stop"),
+        since="1.3.0",
+        removed_in="1.5.0",
+        substitute="PagingList[start:stop]",
+    )
+    def list_branches(self, **kwargs: int) -> PagingList[Branch]:
         """List the information of branches.
 
         Arguments:
-            start: The index to start.
-            stop: The index to end.
+            kwargs: For deprecated keyword arguments: "start" and "stop".
 
-        Yields:
-            The :class:`branches<.Branch>`.
+        Returns:
+            The PagingList of :class:`branches<.Branch>`.
 
         """
-        yield from self._list_branches(start=start, stop=stop)
+        start = kwargs.get("start", 0)
+        stop = kwargs.get("stop", sys.maxsize)
+
+        return PagingList(
+            lambda offset, limit: self._generate_branches(None, offset, limit),
+            128,
+            slice(start, stop),
+        )
 
     def checkout(self, revision: Optional[str] = None, draft_number: Optional[int] = None) -> None:
         """Checkout to commit or draft.
@@ -462,16 +487,30 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
         self._status.check_authority_for_draft()
         self._status.checkout(commit_id=self._commit(message, tag))
 
-    def update_notes(self, *, is_continuous: bool) -> None:
+    def update_notes(
+        self,
+        *,
+        is_continuous: Optional[bool] = None,
+        bin_point_cloud_fields: Union[Iterable[str], None] = ...,  # type: ignore[assignment]
+    ) -> None:
         """Update the notes.
 
         Arguments:
             is_continuous: Whether the data is continuous.
+            bin_point_cloud_fields: The field names of the bin point cloud files in the dataset.
 
         """
         self._status.check_authority_for_draft()
 
-        patch_data: Dict[str, Any] = {"isContinuous": is_continuous}
+        patch_data: Dict[str, Any] = {}
+        if is_continuous is not None:
+            patch_data["isContinuous"] = is_continuous
+
+        if bin_point_cloud_fields is None:
+            patch_data["binPointCloudFields"] = bin_point_cloud_fields
+        elif bin_point_cloud_fields is not ...:  # type: ignore[comparison-overlap]
+            patch_data["binPointCloudFields"] = list(bin_point_cloud_fields)
+
         patch_data.update(self._status.get_status_info())
 
         self._client.open_api_do("PATCH", "notes", self.dataset_id, json=patch_data)
@@ -489,18 +528,26 @@ class DatasetClientBase:  # pylint: disable=too-many-public-methods
             self._client.open_api_do("GET", "notes", self.dataset_id, params=params).json()
         )
 
-    def list_segment_names(self, *, start: int = 0, stop: int = sys.maxsize) -> Iterator[str]:
+    @KwargsDeprecated(
+        ("start", "stop"),
+        since="1.3.0",
+        removed_in="1.5.0",
+        substitute="PagingList[start:stop]",
+    )
+    def list_segment_names(self, **kwargs: int) -> PagingList[str]:
         """List all segment names in a certain commit.
 
         Arguments:
-            start: The index to start.
-            stop: The index to end.
+            kwargs: For deprecated keyword arguments: "start" and "stop".
 
-        Yields:
-            Required segment names.
+        Returns:
+            The PagingList of segment names.
 
         """
-        yield from (segment["name"] for segment in self._list_segments(start=start, stop=stop))
+        start = kwargs.get("start", 0)
+        stop = kwargs.get("stop", sys.maxsize)
+
+        return PagingList(self._generate_segment_names, 128, slice(start, stop))
 
     def get_catalog(self) -> Catalog:
         """Get the catalog of the certain commit.
@@ -573,6 +620,32 @@ class DatasetClient(DatasetClientBase):
 
     """
 
+    def _upload_segment(
+        self,
+        segment: Segment,
+        *,
+        jobs: int = 1,
+        skip_uploaded_files: bool = False,
+        pbar: Tqdm,
+    ) -> SegmentClient:
+        segment_client = self.get_or_create_segment(segment.name)
+        local_data: Iterator[Data] = filter(
+            lambda data: pbar.update_for_skip(isinstance(data, Data)),
+            segment,  # type: ignore[arg-type]
+        )
+        if skip_uploaded_files:
+            done_set = set(segment_client.list_data_paths())
+            segment_filter = filter(
+                lambda data: pbar.update_for_skip(data.target_remote_path not in done_set),
+                local_data,
+            )
+        else:
+            segment_filter = local_data
+
+        multithread_upload(segment_client.upload_data, segment_filter, jobs=jobs, pbar=pbar)
+
+        return segment_client
+
     def get_or_create_segment(self, name: str = "") -> SegmentClient:
         """Get or create a segment with the given name.
 
@@ -632,42 +705,33 @@ class DatasetClient(DatasetClientBase):
         *,
         jobs: int = 1,
         skip_uploaded_files: bool = False,
+        quiet: bool = False,
     ) -> SegmentClient:
         """Upload a :class:`~tensorbay.dataset.segment.Segment` to the dataset.
 
         This function will upload all info contains in
-        the input :class:`~tensorbay.dataset.segment.Segment`,
-        which includes:
-        - Create a segment using the name of input Segment.
-        - Upload all Data in the Segment to the dataset.
+        the input :class:`~tensorbay.dataset.segment.Segment`, which includes:
+
+            - Create a segment using the name of input Segment.
+            - Upload all Data in the Segment to the dataset.
 
         Arguments:
             segment: The :class:`~tensorbay.dataset.segment.Segment`
                 contains the information needs to be upload.
             jobs: The number of the max workers in multi-thread uploading method.
             skip_uploaded_files: True for skipping the uploaded files.
+            quiet: Set to True to stop showing the upload process bar.
 
         Returns:
             The :class:`~tensorbay.client.segment.SegmentClient`
-                used for uploading the data in the segment.
+            used for uploading the data in the segment.
 
         """
         self._status.check_authority_for_draft()
-
-        segment_client = self.get_or_create_segment(segment.name)
-        local_data: Iterator[Data] = filter(
-            lambda data: isinstance(data, Data), segment  # type: ignore[arg-type]
-        )
-        if skip_uploaded_files:
-            done_set = set(segment_client.list_data_paths())
-            segment_filter = filter(
-                lambda data: data.target_remote_path not in done_set, local_data
+        with Tqdm(len(segment), disable=quiet) as pbar:
+            return self._upload_segment(
+                segment, jobs=jobs, skip_uploaded_files=skip_uploaded_files, pbar=pbar
             )
-        else:
-            segment_filter = local_data
-
-        multithread_upload(segment_client.upload_data, segment_filter, jobs=jobs)
-        return segment_client
 
 
 class FusionDatasetClient(DatasetClientBase):
@@ -683,6 +747,45 @@ class FusionDatasetClient(DatasetClientBase):
 
     """
 
+    def _upload_segment(
+        self,
+        segment: FusionSegment,
+        *,
+        jobs: int,
+        skip_uploaded_files: bool,  # pylint: disable=unused-argument
+        pbar: Tqdm,
+    ) -> FusionSegmentClient:
+        segment_client = self.get_or_create_segment(segment.name)
+        for sensor in segment.sensors.values():
+            segment_client.upload_sensor(sensor)
+
+        segment_filter: Iterator[Tuple[Frame, Optional[int]]]
+
+        if not segment:
+            return segment_client
+
+        have_frame_id = hasattr(segment[0], "frame_id")
+
+        for frame in segment:
+            if not hasattr(frame, "frame_id") == have_frame_id:
+                raise FrameError(
+                    "All the frames should have the same patterns(all have frame id or not)."
+                )
+
+        if have_frame_id:
+            segment_filter = ((frame, None) for frame in segment)
+        else:
+            segment_filter = ((frame, 10 * index + 10) for index, frame in enumerate(segment))
+
+        multithread_upload(
+            lambda args: segment_client.upload_frame(*args),
+            segment_filter,
+            jobs=jobs,
+            pbar=pbar,
+        )
+
+        return segment_client
+
     def get_or_create_segment(self, name: str = "") -> FusionSegmentClient:
         """Get or create a fusion segment with the given name.
 
@@ -741,6 +844,7 @@ class FusionDatasetClient(DatasetClientBase):
         *,
         jobs: int = 1,
         skip_uploaded_files: bool = False,  # pylint: disable=unused-argument
+        quiet: bool = False,
     ) -> FusionSegmentClient:
         """Upload a fusion segment object to the draft.
 
@@ -755,9 +859,7 @@ class FusionDatasetClient(DatasetClientBase):
             segment: The :class:`~tensorbay.dataset.segment.FusionSegment`.
             jobs: The number of the max workers in multi-thread upload.
             skip_uploaded_files: Set it to True to skip the uploaded files.
-
-        Raises:
-            TypeError: When all the frames have the same patterns(both have frame id or not).
+            quiet: Set to True to stop showing the upload process bar.
 
         Returns:
             The :class:`~tensorbay.client.segment.FusionSegmentClient`
@@ -765,33 +867,7 @@ class FusionDatasetClient(DatasetClientBase):
 
         """
         self._status.check_authority_for_draft()
-
-        segment_client = self.get_or_create_segment(segment.name)
-        for sensor in segment.sensors.values():
-            segment_client.upload_sensor(sensor)
-
-        segment_filter: Iterator[Tuple[Frame, Optional[int]]]
-
-        if not segment:
-            return segment_client
-
-        have_frame_id = hasattr(segment[0], "frame_id")
-
-        for frame in segment:
-            if not hasattr(frame, "frame_id") == have_frame_id:
-                raise TypeError(
-                    "All the frames should have the same patterns(both have frame id or not)."
-                )
-
-        if have_frame_id:
-            segment_filter = ((frame, None) for frame in segment)
-        else:
-            segment_filter = ((frame, 10 * index + 10) for index, frame in enumerate(segment))
-
-        multithread_upload(
-            lambda args: segment_client.upload_frame(*args),
-            segment_filter,
-            jobs=jobs,
-        )
-
-        return segment_client
+        with Tqdm(len(segment), disable=quiet) as pbar:
+            return self._upload_segment(
+                segment, jobs=jobs, skip_uploaded_files=skip_uploaded_files, pbar=pbar
+            )
