@@ -14,14 +14,16 @@ AccessKey is required when operating with dataset.
 """
 
 import sys
-from typing import Any, Dict, Iterator, Optional, Type, Union, overload
+from typing import Any, Dict, Generator, Optional, Type, Union, overload
 
 from typing_extensions import Literal
 
 from ..dataset import Dataset, FusionDataset
+from ..exception import DatasetTypeError
+from ..utility import KwargsDeprecated
 from .dataset import DatasetClient, FusionDatasetClient
-from .exceptions import GASDatasetError, GASDatasetTypeError
-from .requests import Client, paging_range
+from .exceptions import GASDatasetError
+from .requests import Client, PagingList, Tqdm
 
 DatasetClientType = Union[DatasetClient, FusionDatasetClient]
 
@@ -41,6 +43,19 @@ class GAS:
     def __init__(self, access_key: str, url: str = "") -> None:
         self._client = Client(access_key, url)
 
+    def _generate_auth_storage_configs(
+        self, name: Optional[str] = None, offset: int = 0, limit: int = 128
+    ) -> Generator[Dict[str, Any], None, int]:
+        params: Dict[str, Any] = {"offset": offset, "limit": limit}
+        if name:
+            params["name"] = name
+
+        response = self._client.open_api_do("GET", "auth-storage-configs", "", params=params).json()
+
+        yield from response["configs"]
+
+        return response["totalCount"]  # type: ignore[no-any-return]
+
     def _get_dataset_with_any_type(self, name: str) -> DatasetClientType:
         info = self._get_dataset(name)
         dataset_id = info["id"]
@@ -66,33 +81,81 @@ class GAS:
             raise GASDatasetError(name)
 
         try:
-            info = next(self._list_datasets(name))
-        except StopIteration as error:
+            response = self._list_datasets(name=name)
+            info = response["datasets"][0]
+        except IndexError as error:
             raise GASDatasetError(name) from error
 
-        return info
+        return info  # type: ignore[no-any-return]
 
     def _list_datasets(
         self,
         name: Optional[str] = None,
         need_team_dataset: bool = False,  # personal: False, all: True
-        *,
-        start: int = 0,
-        stop: int = sys.maxsize,
-        page_size: int = 128,
-    ) -> Iterator[Dict[str, Any]]:
-
-        params: Dict[str, Any] = {}
+        offset: int = 0,
+        limit: int = 128,
+    ) -> Dict[str, Any]:
+        params: Dict[str, Any] = {
+            "offset": offset,
+            "limit": limit,
+        }
         if name:
             params["name"] = name
         if need_team_dataset:
             params["needTeamDataset"] = need_team_dataset
 
-        for params["offset"], params["limit"] in paging_range(start, stop, page_size):
-            response = self._client.open_api_do("GET", "", params=params).json()
-            yield from response["datasets"]
-            if response["recordSize"] + response["offset"] >= response["totalCount"]:
-                break
+        response = self._client.open_api_do("GET", "", params=params)
+        return response.json()  # type: ignore[no-any-return]
+
+    def _generate_dataset_names(
+        self,
+        name: Optional[str] = None,
+        need_team_dataset: bool = False,  # personal: False, all: True
+        offset: int = 0,
+        limit: int = 128,
+    ) -> Generator[str, None, int]:
+        response = self._list_datasets(name, need_team_dataset, offset, limit)
+
+        for item in response["datasets"]:
+            yield item["name"]
+
+        return response["totalCount"]  # type: ignore[no-any-return]
+
+    def get_auth_storage_config(self, name: str) -> Dict[str, Any]:
+        """Get the auth storage config with the given name.
+
+        Arguments:
+            name: The required auth storage config name.
+
+        Returns:
+            The auth storage config with the given name.
+
+        Raises:
+            TypeError: When the required auth storage config does not exist
+                 or the given auth storage config is illegal.
+
+        """
+        if not name:
+            raise TypeError("The given auth storage config name is illegal")
+
+        try:
+            config = next(self._generate_auth_storage_configs(name))
+        except StopIteration as error:
+            raise TypeError(f"The auth storage config: {name} does not exist.") from error
+
+        return config
+
+    def list_auth_storage_configs(self) -> PagingList[Dict[str, Any]]:
+        """List auth storage configs.
+
+        Returns:
+            The PagingList of all auth storage configs.
+
+        """
+        return PagingList(
+            lambda offset, limit: self._generate_auth_storage_configs(None, offset, limit),
+            128,
+        )
 
     @overload
     def create_dataset(
@@ -141,8 +204,8 @@ class GAS:
 
         Returns:
             The created :class:`~tensorbay.client.dataset.DatasetClient` instance or
-                :class:`~tensorbay.client.dataset.FusionDatasetClient` instance (is_fusion=True),
-                and the status of dataset client is "commit".
+            :class:`~tensorbay.client.dataset.FusionDatasetClient` instance (is_fusion=True),
+            and the status of dataset client is "commit".
 
         """
         post_data = {
@@ -156,6 +219,35 @@ class GAS:
         ReturnType: Type[DatasetClientType] = FusionDatasetClient if is_fusion else DatasetClient
         return ReturnType(name, response.json()["id"], self)
 
+    def create_auth_dataset(
+        self, name: str, config_name: str, path: str, is_fusion: bool = False
+    ) -> DatasetClientType:
+        """Create a TensorBay dataset with given name in auth cloud storage.
+
+        The dataset will be linked to the given auth cloud storage
+            and all of relative data will be stored in auth cloud storage.
+
+        Arguments:
+            name: Name of the dataset, unique for a user.
+            config_name: The auth storage config name.
+            path: The path of the dataset to create in auth cloud storage.
+            is_fusion: Whether the dataset is a fusion dataset, True for fusion dataset.
+
+        Returns:
+            The created :class:`~tensorbay.client.dataset.DatasetClient` instance or
+            :class:`~tensorbay.client.dataset.FusionDatasetClient` instance (is_fusion=True),
+            and the status of dataset client is "commit".
+
+        """
+        post_data = {
+            "name": name,
+            "type": int(is_fusion),  # normal dataset: 0, fusion dataset: 1
+            "storageConfig": {"name": config_name, "path": path},
+        }
+        response = self._client.open_api_do("POST", "", json=post_data)
+        ReturnType: Type[DatasetClientType] = FusionDatasetClient if is_fusion else DatasetClient
+        return ReturnType(name, response.json()["id"], self)
+
     @overload
     def get_dataset(self, name: str, is_fusion: Literal[False] = False) -> DatasetClient:
         ...
@@ -177,11 +269,11 @@ class GAS:
 
         Returns:
             The requested :class:`~tensorbay.client.dataset.DatasetClient` instance or
-                :class:`~tensorbay.client.dataset.FusionDatasetClient` instance (is_fusion=True),
-                and the status of dataset client is "commit".
+            :class:`~tensorbay.client.dataset.FusionDatasetClient` instance (is_fusion=True),
+            and the status of dataset client is "commit".
 
         Raises:
-            GASDatasetTypeError: When the requested dataset type is not the same as given.
+            DatasetTypeError: When the requested dataset type is not the same as given.
 
         """
         info = self._get_dataset(name)
@@ -191,22 +283,34 @@ class GAS:
         commit_id = info["HEAD"]["commitId"]
 
         if is_fusion != type_flag:
-            raise GASDatasetTypeError(name, type_flag)
+            raise DatasetTypeError(name, type_flag)
         ReturnType: Type[DatasetClientType] = FusionDatasetClient if is_fusion else DatasetClient
         return ReturnType(name, dataset_id, self, commit_id=commit_id)
 
-    def list_dataset_names(self, *, start: int = 0, stop: int = sys.maxsize) -> Iterator[str]:
+    @KwargsDeprecated(
+        ("start", "stop"),
+        since="1.3.0",
+        removed_in="1.5.0",
+        substitute="PagingList[start:stop]",
+    )
+    def list_dataset_names(self, **kwargs: int) -> PagingList[str]:
         """List names of all TensorBay datasets.
 
         Arguments:
-            start: The index to start.
-            stop: The index to stop.
+            kwargs: For deprecated keyword arguments: "start" and "stop".
 
-        Yields:
-            Names of all datasets.
+        Returns:
+            The PagingList of all TensorBay dataset names.
 
         """
-        yield from (item["name"] for item in self._list_datasets(start=start, stop=stop))
+        start = kwargs.get("start", 0)
+        stop = kwargs.get("stop", sys.maxsize)
+
+        return PagingList(
+            lambda offset, limit: self._generate_dataset_names(None, False, offset, limit),
+            128,
+            slice(start, stop),
+        )
 
     def rename_dataset(self, name: str, new_name: str) -> None:
         """Rename a TensorBay Dataset with given name.
@@ -228,6 +332,7 @@ class GAS:
         *,
         jobs: int = 1,
         skip_uploaded_files: bool = False,
+        quiet: bool = False,
     ) -> DatasetClient:
         ...
 
@@ -239,6 +344,7 @@ class GAS:
         *,
         jobs: int = 1,
         skip_uploaded_files: bool = False,
+        quiet: bool = False,
     ) -> FusionDatasetClient:
         ...
 
@@ -250,6 +356,7 @@ class GAS:
         *,
         jobs: int = 1,
         skip_uploaded_files: bool = False,
+        quiet: bool = False,
     ) -> DatasetClientType:
         ...
 
@@ -260,6 +367,7 @@ class GAS:
         *,
         jobs: int = 1,
         skip_uploaded_files: bool = False,
+        quiet: bool = False,
     ) -> DatasetClientType:
         """Upload a local dataset to TensorBay.
 
@@ -269,7 +377,7 @@ class GAS:
 
             - Create a TensorBay dataset with the name and type of input local dataset.
             - Upload all :class:`~tensorbay.dataset.segment.Segment`
-                or :class:`~tensorbay.dataset.segment.FusionSegment` in the dataset to TensorBay.
+              or :class:`~tensorbay.dataset.segment.FusionSegment` in the dataset to TensorBay.
 
         Arguments:
             dataset: The :class:`~tensorbay.dataset.dataset.Dataset` or
@@ -277,11 +385,12 @@ class GAS:
             jobs: The number of the max workers in multi-thread upload.
             skip_uploaded_files: Set it to True to skip the uploaded files.
             draft_number: The draft number.
+            quiet: Set to True to stop showing the upload process bar.
 
         Returns:
             The :class:`~tensorbay.client.dataset.DatasetClient` or
-                :class:`~tensorbay.client.dataset.FusionDatasetClient`
-                bound with the uploaded dataset.
+            :class:`~tensorbay.client.dataset.FusionDatasetClient`
+            bound with the uploaded dataset.
 
         """
         dataset_client = self.get_dataset(dataset.name, isinstance(dataset, FusionDataset))
@@ -295,12 +404,14 @@ class GAS:
 
         dataset_client.update_notes(**dataset.notes)  # type: ignore[arg-type]
 
-        for segment in dataset:
-            dataset_client.upload_segment(
-                segment,  # type: ignore[arg-type]
-                jobs=jobs,
-                skip_uploaded_files=skip_uploaded_files,
-            )
+        with Tqdm(sum(len(segment) for segment in dataset), disable=quiet) as pbar:
+            for segment in dataset:
+                dataset_client._upload_segment(  # pylint: disable=protected-access
+                    segment,  # type: ignore[arg-type]
+                    jobs=jobs,
+                    skip_uploaded_files=skip_uploaded_files,
+                    pbar=pbar,
+                )
 
         return dataset_client
 
