@@ -15,25 +15,50 @@ import logging
 import os
 from collections import defaultdict
 from concurrent.futures import FIRST_EXCEPTION, ThreadPoolExecutor, wait
-from itertools import repeat, zip_longest
-from typing import Any, Callable, DefaultDict, Iterable, Iterator, Optional, Tuple, TypeVar
+from itertools import count
+from threading import Lock
+from typing import (
+    Any,
+    Callable,
+    DefaultDict,
+    Dict,
+    Generator,
+    Iterable,
+    Optional,
+    Sequence,
+    TypeVar,
+    Union,
+    overload,
+)
 from urllib.parse import urljoin
 
 from requests import Session
 from requests.adapters import HTTPAdapter
 from requests.exceptions import RequestException
 from requests.models import PreparedRequest, Response
+from tqdm import tqdm
 from urllib3.util.retry import Retry
 
 from ..__verison__ import __version__
-from .exceptions import GASResponseError
+from ..exception import ResponseError
+from ..utility import ReprMixin, ReprType
 from .log import RequestLogging, ResponseLogging
 
 logger = logging.getLogger(__name__)
 
 
 class Config:  # pylint: disable=too-few-public-methods
-    """This is a base class defining the concept of Request Config."""
+    """This is a base class defining the concept of Request Config.
+
+    Attributes:
+        max_retries: Maximum retry times of the request.
+        allowed_retry_methods: The allowed methods for retrying request.
+        allowed_retry_status: The allowed status for retrying request.
+            If both methods and status are fitted, the retrying strategy will work.
+        timeout: Timeout value of the request in seconds.
+        is_internal: Whether the request is from internal.
+
+    """
 
     def __init__(self) -> None:
 
@@ -42,21 +67,11 @@ class Config:  # pylint: disable=too-few-public-methods
         self.allowed_retry_status = [429, 500, 502, 503, 504]
 
         self.timeout = 30
-        self._is_intern = False
+        self.is_internal = False
         self._x_source = "PYTHON-SDK"
 
-    @property
-    def is_intern(self) -> bool:
-        """Get whether the request is from intern.
 
-        Returns:
-            Whether the request is from intern.
-
-        """
-        return self._is_intern
-
-
-default_config = Config()
+config = Config()
 
 
 class TimeoutHTTPAdapter(HTTPAdapter):
@@ -70,7 +85,7 @@ class TimeoutHTTPAdapter(HTTPAdapter):
     """
 
     def __init__(self, *args: Any, timeout: Optional[int] = None, **kwargs: Any) -> None:
-        self.timeout = timeout if timeout is not None else default_config.timeout
+        self.timeout = timeout if timeout is not None else config.timeout
         super().__init__(*args, **kwargs)
 
     def send(  # pylint: disable=too-many-arguments
@@ -110,9 +125,9 @@ class UserSession(Session):  # pylint: disable=too-few-public-methods
         # self.session.hooks["response"] = [logging_hook]
 
         retry_strategy = Retry(
-            total=default_config.max_retries,
-            status_forcelist=default_config.allowed_retry_status,
-            method_whitelist=default_config.allowed_retry_methods,
+            total=config.max_retries,
+            status_forcelist=config.allowed_retry_status,
+            method_whitelist=config.allowed_retry_methods,
             raise_on_status=False,
         )
 
@@ -134,14 +149,16 @@ class UserSession(Session):  # pylint: disable=too-few-public-methods
             Response of the request.
 
         Raises:
-            GASResponseError: If post response error.
+            ResponseError: If post response error.
 
         """
         try:
             response = super().request(method, url, *args, **kwargs)
             if response.status_code not in (200, 201):
-                logger.error("Invalid state code!%s", ResponseLogging(response))
-                raise GASResponseError(response)
+                logger.error(
+                    "Unexpected status code(%d)!%s", response.status_code, ResponseLogging(response)
+                )
+                raise ResponseError(response)
             logger.debug(ResponseLogging(response))
             return response
 
@@ -207,7 +224,10 @@ class Client:
             else:
                 url = urljoin(dataset_url, dataset_id)
         else:
-            url = urljoin(self._open_api, "datasets")
+            if section:
+                url = urljoin(self._open_api, section)
+            else:
+                url = urljoin(self._open_api, "datasets")
         return url
 
     @property
@@ -237,7 +257,7 @@ class Client:
         kwargs.setdefault("headers", {})["X-Token"] = self.access_key
         kwargs["headers"][
             "X-Source"
-        ] = f"{default_config._x_source}/{__version__}"  # pylint: disable=protected-access
+        ] = f"{config._x_source}/{__version__}"  # pylint: disable=protected-access
 
         return self.do(method=method, url=self._url_make(section, dataset_id), **kwargs)
 
@@ -259,11 +279,44 @@ class Client:
 _T = TypeVar("_T")
 
 
+class Tqdm(tqdm):  # type: ignore[misc]
+    """A wrapper class of tqdm for showing the process bar.
+
+    Arguments:
+        total: The number of excepted iterations.
+        disable: Whether to disable the entire progress bar.
+
+    """
+
+    def __init__(self, total: int, disable: bool = False) -> None:
+        super().__init__(desc="Uploading", total=total, disable=disable)
+
+    def update_callback(self, _: Any) -> None:
+        """Callback function for updating process bar when multithread task is done."""
+        self.update()
+
+    def update_for_skip(self, condition: bool) -> bool:
+        """Update process bar for the items which are skipped in builtin filter function.
+
+        Arguments:
+            condition: The filter condition, the process bar will be updated if condition is False.
+
+        Returns:
+            The input condition.
+
+        """
+        if not condition:
+            self.update()
+
+        return condition
+
+
 def multithread_upload(
     function: Callable[[_T], None],
     arguments: Iterable[_T],
     *,
     jobs: int = 1,
+    pbar: Tqdm,
 ) -> None:
     """Multi-thread upload framework.
 
@@ -271,10 +324,13 @@ def multithread_upload(
         function: The upload function.
         arguments: The arguments of the upload function.
         jobs: The number of the max workers in multi-thread uploading procession.
+        pbar: The :class:`Tqdm` instance for showing the upload process bar.
 
     """
     with ThreadPoolExecutor(jobs) as executor:
         futures = [executor.submit(function, argument) for argument in arguments]
+        for future in futures:
+            future.add_done_callback(pbar.update_callback)
 
         done, not_done = wait(futures, return_when=FIRST_EXCEPTION)
         for future in not_done:
@@ -283,24 +339,129 @@ def multithread_upload(
             future.result()
 
 
-def paging_range(start: int, stop: int, limit: int) -> Iterator[Tuple[int, int]]:
-    """A Generator which generates offset and limit for paging request.
+class PagingList(Sequence[_T], ReprMixin):  # pylint: disable=too-many-ancestors
+    """PagingList is a wrap of web paging request.
 
-    Examples:
-        >>> paging_range(0, 10, 3)
-        <generator object paging_range at 0x11b9932e0>
-
-        >>> list(paging_range(0, 10, 3))
-        [(0, 3), (3, 3), (6, 3), (9, 1)]
+    It follows the python Sequence protocal, which means it can be used like a python builtin list.
+    And it provides features like lazy evaluation and cache.
 
     Arguments:
-        start: The paging index to start.
-        stop: The paging index to end.
-        limit: The paging limit.
-
-    Yields:
-        The tuple (offset, limit) for paging request.
+        func: A paging generator function, which inputs offset<int> and limit<int> and returns a
+            generator. The returned generator should yield the element user needs, and return the
+            total count of the elements in the paging request.
+        limit: The page size of each paging request.
+        slicing: The required slice of PagingList.
 
     """
-    div, mod = divmod(stop - start, limit)
-    yield from zip_longest(range(start, stop, limit), repeat(limit, div), fillvalue=mod)
+
+    _S = TypeVar("_S", bound="PagingList[_T]")
+    _K = TypeVar("_K")
+
+    _repr_type = ReprType.SEQUENCE
+
+    def __init__(
+        self,
+        func: Callable[[int, int], Generator[_T, None, int]],
+        limit: int,
+        slicing: slice = slice(0, None),
+    ) -> None:
+        self._data: Dict[int, _T] = {}
+        self._attr: Dict[str, int] = {}
+        self._locks: DefaultDict[Union[int, str], Lock] = defaultdict(Lock)
+        self._func = func
+        self._limit = limit
+
+        if slicing.step is not None:
+            raise ValueError("The step slice is not supported yet.")
+
+        self._slice = slicing
+        self._len: Optional[int] = None
+
+    def __len__(self) -> int:
+        return self._get_len()
+
+    @overload
+    def __getitem__(self: _S, index: int) -> _T:
+        ...
+
+    @overload
+    def __getitem__(self: _S, index: slice) -> _S:
+        ...
+
+    def __getitem__(self: _S, index: Union[int, slice]) -> Union[_T, _S]:
+        if isinstance(index, slice):
+            return self._get_slice(index)
+
+        index = self._make_index_positive(index)
+        if index < 0 or index >= self._get_len(index):
+            raise IndexError("list index out of range")
+
+        paging_index = self._slice.start + index
+        if paging_index not in self._data:
+            offset = self._get_offset(paging_index)
+            self._call_with_lock(offset, self._extend, offset)
+
+        return self._data[paging_index]
+
+    def _extend(self, offset: int) -> int:
+        generator = self._func(offset, self._limit)
+        try:
+            for i in count(offset):
+                data = next(generator)
+                self._data[i] = data
+        except StopIteration as error:
+            return error.value  # type: ignore[no-any-return]
+
+        raise TypeError("Impossible to be here, add this to make pylint and mypy happy")
+
+    def _get_total_count(self, index: int) -> None:
+        offset = self._get_offset(index)
+        self._attr["totalCount"] = self._extend(offset)
+
+    def _get_len(self, index: int = 0) -> int:
+        if self._len is None:
+            if "totalCount" not in self._attr:
+                self._call_with_lock("totalCount", self._get_total_count, index)
+
+            total_count = self._attr["totalCount"]
+            stop = total_count if self._slice.stop is None else min(total_count, self._slice.stop)
+            start = min(total_count, self._slice.start)
+            self._len = stop - start
+
+        return self._len
+
+    def _make_index_positive(self, index: int) -> int:
+        return index if index >= 0 else self._get_len() + index
+
+    def _get_offset(self, index: int) -> int:
+        return index // self._limit * self._limit
+
+    def _call_with_lock(self, key: Union[int, str], func: Callable[[_K], Any], arg: _K) -> None:
+        lock = self._locks[key]
+        acquire = lock.acquire(blocking=False)
+        try:
+            if acquire:
+                func(arg)
+                del self._locks[key]
+            else:
+                lock.acquire()
+        finally:
+            lock.release()
+
+    def _get_slice(self: _S, input_slice: slice) -> _S:
+        start = self._slice.start
+        if input_slice.start is not None:
+            start += self._make_index_positive(input_slice.start)
+
+        stop = input_slice.stop
+        if stop is not None:
+            stop = self._make_index_positive(stop) + self._slice.start
+
+        paging_list = self.__class__(self._func, self._limit, slice(start, stop))
+
+        # The sliced PagingList shares the "_data", "_attr" and "_locks" with the root PagingList
+        paging_list._data = self._data  # pylint: disable=protected-access
+        paging_list._attr = self._attr  # pylint: disable=protected-access
+        paging_list._locks = self._locks  # pylint: disable=protected-access
+
+        return paging_list
