@@ -8,9 +8,10 @@
 import pytest
 
 from tensorbay import __version__
-from tensorbay.client import GAS, GASResponseError, GASSegmentError
+from tensorbay.client import GAS, GASSegmentError
 from tensorbay.client.struct import Draft
 from tensorbay.dataset import Data, Frame, FusionSegment, Segment
+from tensorbay.exception import CommitStatusError, ResponseError
 from tensorbay.label import Catalog, Label
 from tensorbay.sensor import Sensor
 
@@ -80,7 +81,7 @@ class TestDatasetClient:
         assert dataset_client.status.is_draft
         assert dataset_client.status.draft_number == draft_number_1
         assert dataset_client.status.commit_id is None
-        with pytest.raises(TypeError):
+        with pytest.raises(CommitStatusError):
             dataset_client.create_draft("draft-2")
         draft_number = get_draft_number_by_title(dataset_client.list_drafts(), "draft-1")
         assert draft_number_1 == draft_number
@@ -99,7 +100,7 @@ class TestDatasetClient:
         with pytest.raises(TypeError):
             get_draft_number_by_title(dataset_client.list_drafts(), "draft-1")
 
-        drafts = list(dataset_client.list_drafts())
+        drafts = dataset_client.list_drafts()
         assert len(drafts) == 1
         assert drafts[0] == Draft(draft_number_2, "draft-2")
 
@@ -118,7 +119,7 @@ class TestDatasetClient:
         dataset_client.commit("commit-2", tag="V1")
 
         dataset_client.create_draft("draft-3")
-        with pytest.raises(GASResponseError):
+        with pytest.raises(ResponseError):
             dataset_client.commit("commit-3", tag="V1")
         dataset_client.commit("commit-3", tag="V2")
         assert not dataset_client.status.is_draft
@@ -137,17 +138,17 @@ class TestDatasetClient:
         dataset_client.create_draft("draft-1")
         dataset_client.commit("commit-1", tag="V1")
         # Can not create more than one tag for one commit
-        with pytest.raises(GASResponseError):
+        with pytest.raises(ResponseError):
             dataset_client.create_tag("V2")
         dataset_client.create_draft("draft-2")
         dataset_client.commit("commit-2")
         # Can not create duplicated tag
-        with pytest.raises(GASResponseError):
+        with pytest.raises(ResponseError):
             dataset_client.create_tag("V1")
         commit_2_id = dataset_client.status.commit_id
         dataset_client.create_draft("draft-3")
         # Can not create the tag without giving commit in the draft
-        with pytest.raises(TypeError):
+        with pytest.raises(CommitStatusError):
             dataset_client.create_tag("V2")
         dataset_client.create_tag("V2", revision=commit_2_id)
         dataset_client.commit("commit-3")
@@ -209,7 +210,7 @@ class TestDatasetClient:
         dataset_client.create_draft("draft-2")
         dataset_client.commit("commit-2", tag="V2")
 
-        tags = list(dataset_client.list_tags())
+        tags = dataset_client.list_tags()
         assert tags[0].name == "V2"
         assert tags[1].name == "V1"
 
@@ -261,7 +262,7 @@ class TestDatasetClient:
 
         commit_1_id = dataset_client.status.commit_id
 
-        branches = list(dataset_client.list_branches())
+        branches = dataset_client.list_branches()
         assert len(branches) == 1
         assert branches[0].name == "main"
         assert branches[0].commit_id == commit_1_id
@@ -303,7 +304,7 @@ class TestDatasetClient:
 
         # Can not create the tag without giving commit in the draft
         dataset_client.create_draft("draft-3")
-        with pytest.raises(TypeError):
+        with pytest.raises(CommitStatusError):
             dataset_client.get_commit()
 
         gas_client.delete_dataset(dataset_name)
@@ -355,21 +356,21 @@ class TestDatasetClient:
         dataset_client.commit("commit-3")
         commit_3_id = dataset_client.status.commit_id
 
-        commits = list(dataset_client.list_commits())
+        commits = dataset_client.list_commits()
         assert len(commits) == 3
         assert commits[0].commit_id == commit_3_id
         assert commits[1].commit_id == commit_2_id
 
-        commits = list(dataset_client.list_commits(commit_2_id))
+        commits = dataset_client.list_commits(commit_2_id)
         assert len(commits) == 2
         assert commits[0].commit_id == commit_2_id
         assert commits[1].commit_id == commit_1_id
 
-        commits = list(dataset_client.list_commits("V1"))
+        commits = dataset_client.list_commits("V1")
         assert len(commits) == 1
         assert commits[0].commit_id == commit_1_id
 
-        commits = list(dataset_client.list_commits("main"))
+        commits = dataset_client.list_commits("main")
         assert len(commits) == 3
 
         gas_client.delete_dataset(dataset_name)
@@ -415,7 +416,7 @@ class TestDatasetClient:
         dataset_name = get_random_dataset_name()
         dataset_client = gas_client.create_dataset(dataset_name)
 
-        with pytest.raises(TypeError):
+        with pytest.raises(CommitStatusError):
             dataset_client.update_notes(is_continuous=True)
         dataset_client.create_draft("draft-1")
         notes_1 = dataset_client.get_notes()
@@ -425,6 +426,13 @@ class TestDatasetClient:
         notes_2 = dataset_client.get_notes()
         assert notes_2.is_continuous is True
 
+        dataset_client.update_notes(
+            is_continuous=False, bin_point_cloud_fields=["X", "Y", "Z", "Intensity", "Ring"]
+        )
+        notes_2 = dataset_client.get_notes()
+        assert notes_2.is_continuous is False
+        assert notes_2.bin_point_cloud_fields == ["X", "Y", "Z", "Intensity", "Ring"]
+
         gas_client.delete_dataset(dataset_name)
 
     def test_create_segment(self, accesskey, url):
@@ -432,7 +440,7 @@ class TestDatasetClient:
         dataset_name = get_random_dataset_name()
         dataset_client = gas_client.create_dataset(dataset_name)
 
-        with pytest.raises(TypeError):
+        with pytest.raises(CommitStatusError):
             dataset_client.create_segment("segment")
         dataset_client.create_draft("draft-1")
         segment_client = dataset_client.create_segment("segment")
@@ -450,7 +458,7 @@ class TestDatasetClient:
         dataset_name = get_random_dataset_name()
         dataset_client = gas_client.create_dataset(dataset_name)
 
-        with pytest.raises(TypeError):
+        with pytest.raises(CommitStatusError):
             dataset_client.get_or_create_segment("segment")
         dataset_client.create_draft("draft-1")
         segment_client = dataset_client.get_or_create_segment("segment")
@@ -465,7 +473,7 @@ class TestDatasetClient:
         dataset_name = get_random_dataset_name()
         dataset_client = gas_client.create_dataset(dataset_name, is_fusion=True)
 
-        with pytest.raises(TypeError):
+        with pytest.raises(CommitStatusError):
             dataset_client.get_or_create_segment("segment")
         dataset_client.create_draft("draft-1")
         segment_client = dataset_client.get_or_create_segment("segment")
@@ -483,7 +491,7 @@ class TestDatasetClient:
         dataset_client.get_or_create_segment("segment1")
         dataset_client.get_or_create_segment("segment2")
 
-        segments = list(dataset_client.list_segment_names())
+        segments = dataset_client.list_segment_names()
         assert "segment1" in segments
         assert "segment2" in segments
         assert "segment3" not in segments
@@ -495,7 +503,7 @@ class TestDatasetClient:
         dataset_name = get_random_dataset_name()
         dataset_client = gas_client.create_dataset(dataset_name)
 
-        with pytest.raises(TypeError):
+        with pytest.raises(CommitStatusError):
             dataset_client.upload_catalog(Catalog.loads(CATALOG))
         dataset_client.create_draft("draft-1")
         dataset_client.upload_catalog(Catalog.loads(CATALOG))
