@@ -7,8 +7,9 @@
 
 import pytest
 
-from tensorbay.client import GAS, GASDatasetError, GASResponseError
+from tensorbay.client import GAS, GASDatasetError
 from tensorbay.dataset import Data, Dataset, Segment
+from tensorbay.exception import ResponseError
 from tensorbay.label import Catalog, Label
 
 from .utility import get_random_dataset_name
@@ -83,7 +84,7 @@ class TestGAS:
 
         region = "guangzhou"
         dataset_name = get_random_dataset_name()
-        with pytest.raises(GASResponseError):
+        with pytest.raises(ResponseError):
             gas_client.create_dataset(dataset_name, region=region)
 
     def test_create_fusion_dataset(self, accesskey, url):
@@ -105,7 +106,7 @@ class TestGAS:
         dataset_name = get_random_dataset_name()
         gas_client.create_dataset(dataset_name)
 
-        datasets = list(gas_client.list_dataset_names())
+        datasets = gas_client.list_dataset_names()
         assert dataset_name in datasets
         dataset_name_1 = get_random_dataset_name()
         assert dataset_name_1 not in datasets
